@@ -1,4 +1,5 @@
 mixed_model <- function (fixed, random, data, family, na.action = na.exclude,
+                         zi_fixed = NULL, zi_random = NULL, 
                          penalized = FALSE, n_phis = NULL, initial_values = NULL, 
                          control = list(), ...) {
     call <- match.call()
@@ -21,8 +22,14 @@ mixed_model <- function (fixed, random, data, family, na.action = na.exclude,
     }
     known_families <- c("binomial", "poisson", "negative binomial")
     # extract response vector, design matrices, offset
+    data <- as.data.frame(data) # in case 'data' is a tibble
     mfX <- model.frame(terms(fixed, data = data), data = data, na.action = na.action)
     na_exclude <- attr(mfX, "na.action")
+    if (!is.null(zi_fixed)) { 
+        mfX_zi <- model.frame(terms(zi_fixed, data = data), data = data, 
+                              na.action = na.action)
+        na_exclude <- union(attr(mfX_zi, "na.action"), na_exclude)
+    }
     termsX <- terms(mfX)
     y <- model.response(mfX)
     if (is.factor(y)) {
@@ -45,12 +52,40 @@ mixed_model <- function (fixed, random, data, family, na.action = na.exclude,
         id_orig <- id_orig[-na_exclude]
     id <- match(id_orig, unique(id_orig))
     ###########################
+    # Zero inflation part
+    if (!is.null(zi_fixed)) {
+        if (!is.null(na_exclude)) 
+            mfX_zi <- mfX_zi[-na_exclude, ]
+        termsX_zi <- terms(mfX_zi)
+        X_zi <- model.matrix(termsX_zi, mfX_zi)
+        offset_zi <- model.offset(mfX_zi)
+    } else {
+        X_zi <- offset_zi <- termsX_zi <- mfX_zi <- NULL
+    }
+    if (!is.null(zi_random)) {
+        form_random_zi <- getRE_Formula(zi_random)
+        mfZ_zi <- model.frame(terms(form_random_zi, data = data), data = data)
+        if (!is.null(na_exclude)) 
+            mfZ_zi <- mfZ_zi[-na_exclude, ]
+        termsZ_zi <- terms(mfZ_zi)
+        Z_zi <- model.matrix(termsZ_zi, mfZ_zi)
+        id_nam_zi <- all.vars(getID_Formula(zi_random))
+        if (id_nam_zi != id_nam) {
+            stop("the 'random' and 'zi_random' formulas for the random effects ",
+                 "should have the same grouping variable.")
+        }
+    } else {
+        Z_zi <- termsZ_zi <- mfZ_zi <- NULL
+    }
+    nRE <- ncol(Z) + if (!is.null(Z_zi)) ncol(Z_zi) else 0
+    ###########################
     # control settings
     con <- list(iter_EM = 30, iter_qN_outer = 15, iter_qN = 10, iter_qN_incr = 10,
                 optim_method = "BFGS", parscale_betas = 0.1, parscale_D = 0.01,
-                parscale_phis = 0.01, tol1 = 1e-03, tol2 = 1e-04, tol3 = 1e-07,
-                numeric_deriv = "fd", nAGQ = if (ncol(Z) < 3) 11 else 7, 
-                update_GH_every = 10, verbose = FALSE)
+                parscale_phis = 0.01, parscale_gammas = 0.01, tol1 = 1e-03, tol2 = 1e-04, 
+                tol3 = 1e-07, numeric_deriv = "fd", nAGQ = if (nRE < 3) 11 else 7, 
+                update_GH_every = 10, max_coef_value = 10, max_phis_value = exp(10), 
+                verbose = FALSE)
     control <- c(control, list(...))
     namC <- names(con)
     con[(namc <- names(control))] <- control
@@ -62,23 +97,23 @@ mixed_model <- function (fixed, random, data, family, na.action = na.exclude,
     inits <- if (family$family %in% known_families || (is.list(initial_values) &&
                  inherits(initial_values$betas, 'family'))) {
         betas <- if (family$family %in% known_families) {
-            glm.fit(X, y, family = family)$coefficients
+            if (family$family == "negative binomial")
+                glm.fit(X, y, family = poisson())$coefficients
+            else 
+                glm.fit(X, y, family = family)$coefficients
         } else {
             glm.fit(X, y, family = initial_values$betas)$coefficients
         }
-        list(betas = betas * sqrt(1.346), D = if (diag_D) rep(1, ncol(Z)) else diag(ncol(Z)))
+        list(betas = betas * sqrt(1.346), D = if (diag_D) rep(1, nRE) else diag(nRE))
     } else {
-        list(betas = rep(0, ncol(X)), D = if (diag_D) rep(1, ncol(Z)) else diag(ncol(Z)))
+        list(betas = rep(0, ncol(X)), D = if (diag_D) rep(1, nRE) else diag(nRE))
     }
-    if (!is.null(initial_values) && is.list(initial_values) &&
-        !inherits(initial_values$betas, 'family')) {
-        lngths <- lapply(inits[(nams.initial_values <- names(initial_values))], length)
-        if (!isTRUE(all.equal(lngths, lapply(initial_values, length)))) {
-            warning("'initial_values' is not a list with elements of appropriate ",
-                    "length; default initial_values are used instead.\n")
-        } else {
-            inits[nams.initial_values] <- initial_values
-        }
+    if (!is.null(zi_fixed)) {
+        inits <- c(inits, 
+                   list(gammas = glm.fit(X_zi, as.numeric(y == 0), 
+                                         family = binomial())$coefficients))
+        if (family$family %in% c("zero-inflated poisson", "zero-inflated negative binomial"))
+            inits$betas <- glm.fit(X, y, family = poisson())$coefficients
     }
     ##########################
     # penalized
@@ -101,11 +136,13 @@ mixed_model <- function (fixed, random, data, family, na.action = na.exclude,
         var_fun = family$variance,
         mu.eta_fun = family$mu.eta
     )
-    if (family$family %in% known_families && is.null(family$log_den)) {
+    if (family$family %in% known_families && is.null(family$log_dens)) {
         Funs$log_dens <- switch(family$family,
                'binomial' = binomial_log_dens,
                'poisson' = poisson_log_dens,
                'negative binomial' = negative.binomial_log_dens)
+    } else if (family$family %in% known_families && !is.null(family$log_dens)) {
+        Funs$log_dens <- family$log_dens
     } else if (!family$family %in% known_families && !is.null(family$log_dens)) {
         Funs$log_dens <- family$log_dens
     } else {
@@ -123,47 +160,78 @@ mixed_model <- function (fixed, random, data, family, na.action = na.exclude,
     if (!is.null(family$score_eta_fun) && is.function(family$score_eta_fun)) {
         Funs$score_eta_fun <- family$score_eta_fun
     }
+    if (!is.null(family$score_eta_zi_fun) && is.function(family$score_eta_zi_fun)) {
+        Funs$score_eta_zi_fun <- family$score_eta_zi_fun
+    }
     if (!is.null(family$score_phis_fun) && is.function(family$score_phis_fun)) {
         Funs$score_phis_fun <- family$score_phis_fun
     }
-    has_phis <- inherits(try(Funs$log_dens(y, 0, Funs$mu_fun, phis = NULL), TRUE),
+    has_phis <- inherits(try(Funs$log_dens(y, rep(0, length(y)), Funs$mu_fun, 
+                                           phis = NULL, rep(0, length(y))), TRUE),
                          "try-error")
     if (has_phis) {
-        if (family$family == "negative binomial") {
+        if (family$family %in% c("negative binomial", "zero-inflated negative binomial")) {
             n_phis <- 1
         } else if (is.null(n_phis)) {
             stop("argument 'n_phis' needs to be specified.\n")
         }
         inits$phis <- rep(0.0, n_phis)
     }
+    if (!is.null(initial_values) && is.list(initial_values) &&
+        !inherits(initial_values$betas, 'family')) {
+        lngths <- lapply(inits[(nams.initial_values <- names(initial_values))], length)
+        if (!isTRUE(all.equal(lngths, lapply(initial_values, length)))) {
+            warning("'initial_values' is not a list with elements of appropriate ",
+                    "length; default initial_values are used instead.\n")
+        } else {
+            inits[nams.initial_values] <- initial_values
+        }
+    }
     ###############
     # Fit the model
-    out <- mixed_fit(y, X, Z, id, offset, family, inits, Funs, con, penalized)
+    out <- mixed_fit(y, X, Z, X_zi, Z_zi, id, offset, offset_zi, family, inits, Funs, 
+                     con, penalized)
+    # check whether Hessian is positive definite at convergence
+    H <- out$Hessian
+    if (any(is.na(H) | !is.finite(H))) {
+        warning("infinite or missing values in Hessian at convergence.\n")
+    } else {
+        ev <- eigen(H, symmetric = TRUE, only.values = TRUE)$values
+        if (!all(ev >= -1e-06 * abs(ev[1L]))) 
+            warning("Hessian matrix at convergence is not positive definite; ", 
+                    "unstable solution.\n")
+    }
     # fix names
     names(out$coefficients) <- colnames(X)
-    dimnames(out$D) <- list(colnames(Z), colnames(Z))
+    RE_nams <- c(colnames(Z), if (!is.null(Z_zi)) paste0("zi_", colnames(Z_zi)))
+    dimnames(out$D) <- list(RE_nams, RE_nams)
     if (!is.null(out$phis))
         names(out$phis) <- paste0("phi_", seq_along(out$phis))
+    if (!is.null(out$gammas))
+        names(out$gammas) <- colnames(X_zi)
     all_nams <- if (diag_D) {
-        nams_D <- paste0("D_", seq_len(ncol(Z)), seq_len(ncol(Z)))
-        c(names(out$coefficients), nams_D, names(out$phis))
+        nams_D <- paste0("D_", seq_len(nRE), seq_len(nRE))
+        c(names(out$coefficients), nams_D, names(out$phis), 
+          if (!is.null(out$gammas)) paste0("zi_", names(out$gammas)))
     } else {
         nams_D <- paste0("D_", apply(which(upper.tri(out$D, TRUE), arr.ind = TRUE), 1, 
                                      paste0, collapse = ""))
-        c(names(out$coefficients), nams_D, names(out$phis))
+        c(names(out$coefficients), nams_D, names(out$phis), 
+          if (!is.null(out$gammas)) paste0("zi_", names(out$gammas)))
     }
     dimnames(out$Hessian) <- list(all_nams, all_nams)
     out$id <- id_orig
     out$id_name <- id_nam 
     out$offset <- offset
-    dimnames(out$post_modes) <- list(unique(id_orig), colnames(Z))
+    dimnames(out$post_modes) <- list(unique(id_orig), RE_nams)
     names(out$post_vars) <- unique(id_orig)
     out$post_vars[] <- lapply(out$post_vars, function (v) {
-        dimnames(v) <- list(colnames(Z), colnames(Z))
+        dimnames(v) <- list(RE_nams, RE_nams)
         v
     })
-    out$Terms <- list(termsX = termsX, termsZ = termsZ)
-    out$model_frames <- list(mfX = mfX, mfZ = mfZ)
+    out$Terms <- list(termsX = termsX, termsZ = termsZ, termsX_zi = termsX_zi, 
+                      termsZ_zi = termsZ_zi)
+    out$model_frames <- list(mfX = mfX, mfZ = mfZ, mfX_zi = mfX_zi, mfZ_zi = mfZ_zi)
     out$control <- con
     out$Funs <- Funs
     out$family <- family
