@@ -94,11 +94,24 @@ xyplot(expit(pred) + expit(low) + expit(upp) ~ time | sex, data = plot_data,
        xlab = "Follow-up time", ylab = "Subject-Specific Probabilities")
 
 ## ---- eval = FALSE-------------------------------------------------------
-#  plot_data_m <- effectPlotData(fm, nDF, marginal = TRUE)
+#  plot_data_m <- effectPlotData(fm, nDF, marginal = TRUE, cores = 4)
 #  
-#  xyplot(expit(pred) + expit(low) + expit(upp) ~ time | sex, data = plot_data_m,
-#         type = "l", lty = c(1, 2, 2), col = c(2, 1, 1), lwd = 2,
-#         xlab = "Follow-up time", ylab = "Marginal Probabilities")
+#  # we put the two groups in the same panel
+#  my.panel.bands <- function(x, y, upper, lower, fill, col, subscripts, ..., font,
+#                             fontface) {
+#      upper <- upper[subscripts]
+#      lower <- lower[subscripts]
+#      panel.polygon(c(x, rev(x)), c(upper, rev(lower)), col = fill, border = FALSE, ...)
+#  }
+#  
+#  xyplot(expit(pred) ~ time, group = sex, data = plot_data_m,
+#         upper = expit(plot_data_m$upp), low = expit(plot_data_m$low),
+#         type = "l", col = c("blue", "red"),
+#         fill = c("#0000FF80", "#FF000080"),
+#         panel = function (x, y, ...) {
+#             panel.superpose(x, y, panel.groups = 'my.panel.bands', ...)
+#             panel.xyplot(x, y, lwd = 2,  ...)
+#  }, xlab = "Follow-up time", ylab = "Marginal Probabilities")
 
 ## ------------------------------------------------------------------------
 gm <- mixed_model(fixed = y ~ sex * time, random = ~ time || id, data = DF,
@@ -106,3 +119,35 @@ gm <- mixed_model(fixed = y ~ sex * time, random = ~ time || id, data = DF,
 
 anova(gm, fm)
 
+## ------------------------------------------------------------------------
+pred_DF <- DF[DF$id == 1, ][1:4, ]
+pred_DF$id <- paste0("N", as.character(pred_DF$id))
+pred_DF
+
+## ------------------------------------------------------------------------
+predict(fm, newdata = pred_DF, type_pred = "response",
+        type = "mean_subject", se.fit = TRUE)
+
+## ---- eval = FALSE-------------------------------------------------------
+#  predict(fm, newdata = pred_DF, type_pred = "response",
+#          type = "marginal", se.fit = FALSE)
+
+## ------------------------------------------------------------------------
+predict(fm, newdata = pred_DF, type_pred = "response",
+        type = "subject_specific", se.fit = TRUE)
+
+## ------------------------------------------------------------------------
+future_Times <- pred_DF[1:3, c("id", "time", "sex")]
+future_Times$time <- c(3, 4, 5)
+future_Times
+
+## ------------------------------------------------------------------------
+predict(fm, newdata = pred_DF, newdata2 = future_Times, type_pred = "response",
+        type = "subject_specific", se.fit = TRUE)
+
+## ------------------------------------------------------------------------
+head(simulate(fm, nsim = 2, seed = 123), 10)
+
+## ------------------------------------------------------------------------
+head(simulate(fm, nsim = 2, seed = 123, acount_MLEs_var = TRUE), 10)
+
